"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertClassTypeDeclaration = void 0;
const t = __importStar(require("@babel/types"));
const convertTSTypeParameterInstantiation_1 = require("./convertTSTypeParameterInstantiation");
const convertTSEntityName_1 = require("./convertTSEntityName");
const convertTSTypeParameterDeclaration_1 = require("./convertTSTypeParameterDeclaration");
const convertTSType_1 = require("./convertTSType");
const convertFunctionTypeAnnotation_1 = require("./convertFunctionTypeAnnotation");
const convertKey_1 = require("./convertKey");
const convertMemberExpressionToQualifiedTypeIdentifier_1 = require("./convertMemberExpressionToQualifiedTypeIdentifier");
function convertClassTypeDeclaration(node) {
    const id = node.id;
    let typeParameters = null;
    if (node.typeParameters) {
        if (!t.isTSTypeParameterDeclaration(node.typeParameters)) {
            throw new Error(`TSTypeParameterDeclaration is expected, but got ${node.typeParameters.type} instead`);
        }
        typeParameters = (0, convertTSTypeParameterDeclaration_1.convertTSTypeParameterDeclaration)(node.typeParameters);
    }
    let _extends = [];
    if (node.superClass) {
        if (!t.isIdentifier(node.superClass) &&
            !t.isMemberExpression(node.superClass)) {
            throw new Error(`Identifier or MemberExpression is expected, but got ${node.superClass.type} instead`);
        }
        if (node.superTypeParameters &&
            !t.isTSTypeParameterInstantiation(node.superTypeParameters)) {
            throw new Error(`TSTypeParameterInstantiation is expected, but got ${node.superClass.type} instead`);
        }
        _extends = [
            t.interfaceExtends((0, convertMemberExpressionToQualifiedTypeIdentifier_1.convertMemberExpressionToQualifiedTypeIdentifier)(node.superClass), node.superTypeParameters
                ? (0, convertTSTypeParameterInstantiation_1.convertTSTypeParameterInstantiation)(node.superTypeParameters)
                : null),
        ];
    }
    const properties = [];
    const indexers = [];
    const callProperties = [];
    const internalSlots = [];
    for (const member of node.body.body) {
        if (t.isClassProperty(member)) {
            const key = (0, convertKey_1.convertKey)(member.key);
            const prop = t.objectTypeProperty(
            // @ts-ignore
            key, t.isTSTypeAnnotation(member.typeAnnotation)
                ? (0, convertTSType_1.convertTSType)(member.typeAnnotation.typeAnnotation)
                : t.anyTypeAnnotation());
            // @ts-expect-error todo: @babel/types
            prop.static = member.static;
            prop.optional = !!member.optional;
            if (member.readonly) {
                prop.variance = t.variance('plus');
            }
            properties.push(prop);
        }
        else if (t.isTSDeclareMethod(member)) {
            const key = t.isIdentifier(member.key) || t.isStringLiteral(member.key)
                ? member.key
                : t.isNumericLiteral(member.key)
                    ? t.stringLiteral(member.key.value + '')
                    : null;
            const { typeParams, parameters, rest, returnType } = (0, convertFunctionTypeAnnotation_1.convertFunctionTypeAnnotation)(member);
            const prop = t.objectTypeProperty(
            // @ts-ignore
            key, t.functionTypeAnnotation(typeParams, parameters, rest, returnType
                ? returnType
                : t.isIdentifier(key) && key.name === 'constructor'
                    ? // special case - constructor type should have return type
                        t.genericTypeAnnotation(t.identifier(id.name))
                    : t.anyTypeAnnotation()));
            // @ts-expect-error todo: @babel/types
            prop.static = member.static;
            prop.kind =
                member.kind === 'get' || member.kind === 'set' ? member.kind : 'init';
            // @ts-ignore
            prop.method = true;
            properties.push(prop);
        }
        else {
            throw new Error('todo:');
        }
    }
    const body = t.objectTypeAnnotation(properties, indexers, callProperties, internalSlots);
    const replacement = t.declareClass(id, typeParameters, _extends, body);
    if (node.implements) {
        replacement.implements = node.implements.map(impl => {
            if (!t.isTSExpressionWithTypeArguments(impl)) {
                throw new Error(`TSExpressionWithTypeArguments is expected, but got ${impl.type} instead`);
            }
            const id = (0, convertTSEntityName_1.convertTSEntityName)(impl.expression);
            const typeParameters = impl.typeParameters
                ? (0, convertTSTypeParameterInstantiation_1.convertTSTypeParameterInstantiation)(impl.typeParameters)
                : null;
            if (!t.isIdentifier(id)) {
                // todo: create alias for the interface to be added to implements
                throw new Error('not implemented, flow does not allow class to implement interface with qualified type name');
            }
            return t.classImplements(id, typeParameters);
        });
    }
    return replacement;
}
exports.convertClassTypeDeclaration = convertClassTypeDeclaration;
//# sourceMappingURL=convertClassTypeDeclaration.js.map